/*
 *
 * Includes for cdc-acm.c
 *
 * Mainly take from usbnet's cdc-ether part
 *
 */

/*
 * CMSPAR, some architectures can't have space and mark parity.
 */

#ifndef CMSPAR
#define CMSPAR			0
#endif

/*
 * Major and minor numbers.
 */

#define ACM_TTY_MAJOR		166
#define ACM_TTY_MINORS		32

/*
 * Requests.
 */

#define USB_RT_ACM		(USB_TYPE_CLASS | USB_RECIP_INTERFACE)

/*
 * Output control lines.
 */

#define ACM_CTRL_DTR		0x01
#define ACM_CTRL_RTS		0x02

/*
 * Input control lines and line errors.
 */

#define ACM_CTRL_DCD		0x01
#define ACM_CTRL_DSR		0x02
#define ACM_CTRL_BRK		0x04
#define ACM_CTRL_RI		0x08

#define ACM_CTRL_FRAMING	0x10
#define ACM_CTRL_PARITY		0x20
#define ACM_CTRL_OVERRUN	0x40

/*
 * Internal driver structures.
 */

/*
 * The only reason to have several buffers is to accomodate assumptions
 * in line disciplines. They ask for empty space amount, receive our URB size,
 * and proceed to issue several 1-character writes, assuming they will fit.
 * The very first write takes a complete URB. Fortunately, this only happens
 * when processing onlcr, so we only need 2 buffers. These values must be
 * powers of 2.
 */
#define ACM_NWU			16
#define ACM_NRU			16
#define ACM_WRITE_OUTSTANDING	16
#define ACM_READ_OUTSTANDING	 8

struct acm {
	struct usb_device *dev;				/* the corresponding usb device */
	struct usb_interface *control;			/* control interface */
	struct usb_interface *data;			/* data interface */
	struct tty_struct *tty;				/* the corresponding tty */
	struct urb ctrlurb;				/* urbs */
	struct urb read_urb[ACM_NRU];
	struct urb write_urb[ACM_NWU];
	u8 *ctrl_buffer,*read_buffer,*write_buffer;	/* buffers of urbs */
	dma_addr_t ctrl_dma,read_dma,write_dma;		/* dma handles of buffers */
	u8 *country_codes;				/* country codes from device */
	unsigned int country_code_size;			/* size of this buffer */
	unsigned int country_rel_date;			/* release date of version */
	unsigned int ctrlsize;
	unsigned int readsize;
	unsigned int writesize;
	spinlock_t write_lock;
	atomic_t write_urb_outstanding;
        __u32 write_urb_used;
        spinlock_t read_lock;
        atomic_t read_urb_outstanding;
        atomic_t in_read_proc;
        unsigned int read_urb_get_idx;
        unsigned int read_urb_put_idx;
	struct usb_cdc_line_coding line;		/* bits, stop, parity */
	struct work_struct work;			/* work queue entry for line discipline waking up */
	struct tasklet_struct urb_task;                 /* rx processing */
	spinlock_t throttle_lock;			/* synchronize throtteling and read callback */
	unsigned int ctrlin;				/* input control lines (DCD, DSR, RI, break, overruns) */
	unsigned int ctrlout;				/* output control lines (DTR, RTS) */
	unsigned int used;				/* someone has this acm's device open */
	unsigned int minor;				/* acm minor number */
	unsigned char throttle;				/* throttled by tty layer */
	unsigned char clocal;				/* termios CLOCAL */
	unsigned int ctrl_caps;				/* control capabilities from the class specific header */
};

#define CDC_DATA_INTERFACE_TYPE	0x0a

#ifndef USB_CDC_CAP_LINE
#define USB_CDC_CAP_LINE	0x02

/* "Country Selection Functional Descriptor" from CDC spec 5.2.3.9 */
struct usb_cdc_country_functional_desc {
	__u8    bLength;
	__u8    bDescriptorType;
	__u8    bDescriptorSubType;

	__u8    iCountryCodeRelDate;
	__le16  wCountyCode0;
	/* ... and there can be a lot of country codes */
} __attribute__ ((packed));
#endif

/* constants describing various quirks and errors */
#define NO_UNION_NORMAL			1
#define SINGLE_RX_URB			2
